@@ -1,4 +1,4 @@
-# Copyright (c) 2012, cPanel, Inc.
+# Copyright (c) 2014, cPanel, Inc.
 # All rights reserved.
 # http://cpanel.net/
 #
@@ -15,11 +15,11 @@ use Filesys::POSIX::Bits::System;
 use Filesys::POSIX::Inode      ();
 use Filesys::POSIX::Mem::Inode ();
 use Filesys::POSIX::IO::Handle ();
+use Filesys::POSIX::Error qw(throw);
 
-use Fcntl qw/:DEFAULT :mode/;
-use Carp qw/confess/;
+use Fcntl qw(:DEFAULT :mode);
 
-our @ISA = qw/Filesys::POSIX::Inode/;
+our @ISA = qw(Filesys::POSIX::Inode);
 
 sub new {
     my ( $class, $path, %opts ) = @_;
@@ -51,7 +51,7 @@ sub new {
 
 sub from_disk {
     my ( $class, $path, %opts ) = @_;
-    my @st = $opts{'st_info'} ? @{ $opts{'st_info'} } : lstat $path or confess($!);
+    my @st = $opts{'st_info'} ? @{ $opts{'st_info'} } : lstat $path or Carp::confess("$!");
 
     my $inode = $class->new( $path, %opts )->update(@st);
 
@@ -66,8 +66,8 @@ sub child {
     my ( $self, $name, $mode ) = @_;
     my $directory = $self->directory;
 
-    confess('Invalid directory entry name') if $name =~ /\//;
-    confess('File exists') if $directory->exists($name);
+    throw &Errno::EINVAL if $name =~ /\//;
+    throw &Errno::EEXIST if $directory->exists($name);
 
     my $path = "$self->{'path'}/$name";
 
@@ -78,17 +78,21 @@ sub child {
     );
 
     if ( ( $mode & $S_IFMT ) == $S_IFREG ) {
-        sysopen( my $fh, $path, O_CREAT | O_EXCL | O_WRONLY, Filesys::POSIX::Bits::System::convertModeToSystem($mode) ) or confess($!);
+        sysopen(
+            my $fh, $path,
+            O_CREAT | O_EXCL | O_WRONLY,
+            Filesys::POSIX::Bits::System::convertModeToSystem($mode)
+        ) or Carp::confess("$!");
         close($fh);
     }
     elsif ( ( $mode & $S_IFMT ) == $S_IFDIR ) {
-        mkdir( $path, $mode ) or confess($!);
+        mkdir( $path, $mode ) or Carp::confess("$!");
     }
 
     my $inode;
 
     if ( ( $mode & $S_IFMT ) == $S_IFLNK ) {
-        confess('Operation not permitted') unless $self->{'sticky'};
+        throw &Errno::EPERM unless $self->{'sticky'};
 
         $inode = Filesys::POSIX::Mem::Inode->new( %data, 'mode' => $mode );
     }
@@ -111,7 +115,7 @@ sub update {
     my ( $self, @st ) = @_;
 
     if ( $self->{'sticky'} && $self->{'dirty'} ) {
-        @{$self}{qw/rdev size atime mtime ctime/} = @st[ 6 .. 10 ];
+        @{$self}{qw(rdev size atime mtime ctime)} = @st[ 6 .. 10 ];
     }
     else {
         $self->SUPER::update(@st);
@@ -123,7 +127,10 @@ sub update {
 sub open {
     my ( $self, $flags ) = @_;
 
-    sysopen( my $fh, $self->{'path'}, Filesys::POSIX::Bits::System::convertFlagsToSystem($flags) ) or confess($!);
+    sysopen(
+        my $fh, $self->{'path'},
+        Filesys::POSIX::Bits::System::convertFlagsToSystem($flags)
+    ) or Carp::confess("$!");
 
     return Filesys::POSIX::IO::Handle->new($fh);
 }
@@ -132,10 +139,10 @@ sub chown {
     my ( $self, $uid, $gid ) = @_;
 
     unless ( $self->{'sticky'} ) {
-        CORE::chown( $uid, $gid, $self->{'path'} ) or confess($!);
+        CORE::chown( $uid, $gid, $self->{'path'} ) or Carp::confess("$!");
     }
 
-    @{$self}{qw/uid gid/} = ( $uid, $gid );
+    @{$self}{qw(uid gid)} = ( $uid, $gid );
 
     return $self->taint;
 }
@@ -146,7 +153,7 @@ sub chmod {
     my $perm = $mode & ( $S_IPERM | $S_IPROT );
 
     unless ( $self->{'sticky'} ) {
-        CORE::chmod( $perm, $self->{'path'} ) or confess($!);
+        CORE::chmod( $perm, $self->{'path'} ) or Carp::confess("$!");
     }
 
     $self->{'mode'} = $format | $perm;
@@ -158,7 +165,7 @@ sub readlink {
     my ($self) = @_;
 
     unless ( $self->{'dest'} ) {
-        $self->{'dest'} = CORE::readlink( $self->{'path'} ) or confess($!);
+        $self->{'dest'} = CORE::readlink( $self->{'path'} ) or Carp::confess("$!");
     }
 
     return $self->{'dest'};
@@ -168,7 +175,7 @@ sub symlink {
     my ( $self, $dest ) = @_;
 
     unless ( $self->{'sticky'} ) {
-        symlink( $dest, $self->{'path'} ) or confess($!);
+        symlink( $dest, $self->{'path'} ) or Carp::confess("$!");
     }
 
     $self->{'dest'} = $dest;
